local addonName = ...

local db, duration
local format = string.format
local floor = math.floor
local GetTime = GetTime


local addon = CreateFrame("Frame", "Broker_AFKTimerDropDown", nil, "UIDropDownMenuTemplate")
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("PLAYER_LEAVING_WORLD")
addon:RegisterEvent("CHAT_MSG_SYSTEM")
addon:SetScript("OnEvent", function(self, event, ...)
	return self[event] and self[event](self, ...)
end)


local function onUpdate(self)
	local counter = ""
	if db.showDuration then
		duration = floor(GetTime() - db.startUptime)
		counter = " ("..addon:SecondsToTime(duration)..")"
	end
	addon.dataobj.text = "AFK since "..db.startTimestamp..counter
end

CreateFrame("Frame"):SetScript("OnUpdate", function(self)
	if not UnitIsAFK("player") then
		db.note = nil
		db.startUptime = nil
		db.startTimestamp = nil
	elseif db.note then
		addon:SetScript("OnUpdate", onUpdate)
	end
	self:SetScript("OnUpdate", nil)
end)


-- dropdown menu items
local dropdown = {
	{
		text = "Broker_AFKTimer",
		isTitle = true,
		notCheckable = true,
	},
	{
		text = "Show duration",
		func = function() db.showDuration = not db.showDuration end,
		checked = function() return db.showDuration end,
		isNotRadio = true,
	},
	{
		text = "Use local time",
		func = function() db.localTime = not db.localTime end,
		checked = function() return db.localTime end,
		isNotRadio = true,
	}
}


-- Broker data object
addon.dataobj = LibStub("LibDataBroker-1.1"):NewDataObject(addonName, {
	type = "data source",
	text = "Not AFK",
	icon = [[Interface\Icons\Spell_Nature_TimeStop]],
	label = "AFK Timer",
	OnClick = function(self, button)
		if button == "LeftButton" then
			SendChatMessage("", "AFK")
		elseif button == "RightButton" then
			EasyMenu(dropdown, addon, self, 0, 0, "MENU")
		end
	end,
	OnTooltipShow = function(self)
		self:AddLine("Broker: AFKTimer", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		self:AddLine(" ")
		self:AddLine(db.note and "You are AFK: |cffffffff"..db.note or "You are not AFK.")
		self:AddLine(" ")
		self:AddLine("Left-click to toggle AFK status.", GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b)
	end,
})


function addon:ADDON_LOADED(addon)
	if addon == addonName then
		-- use saved variables to remember AFK session after a UI reload
		Broker_AFKTimerData = Broker_AFKTimerData or {
			showDuration = true,
			localTime = false,
		}
		db = Broker_AFKTimerData
		self:UnregisterEvent("ADDON_LOADED")
	end
end

function addon:PLAYER_ENTERING_WORLD()
	-- when "entering world" (not on login) AFK is always cancelled, but no system message is sent, so we need a custom made check
	if UnitIsAFK("player") and db.note then
		self:EndAFK()
		DEFAULT_CHAT_FRAME:AddMessage(CLEARED_AFK, 1, 1, 0)
	end
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function addon:PLAYER_LEAVING_WORLD()
	if db.note then
		self:RegisterEvent("PLAYER_ENTERING_WORLD")
	end
end

function addon:CHAT_MSG_SYSTEM(msg)
	-- try to extract an AFK note from the chat system message..
	local note = msg:match(MARKED_AFK_MESSAGE:gsub("$1", ""):gsub("%%s", "(.+)"))
	
	-- ..and if it worked, it was an indication of the player going AFK
	if note then
		local startHr, startMin
		
		-- local or server time?
		if db.localTime then
			local time = date("*t")
			startHr = time.hour
			startMin = time.min
		else
			startHr, startMin = GetGameTime()
		end
		
		db.note = note
		-- get time stamp when AFK starts
		db.startUptime = GetTime() -- get system uptime when AFK starts
		db.startTimestamp = format("%d:%.2d", startHr, startMin)
		duration = 0
		
		self.dataobj.text = "AFK since "..db.startTimestamp..(db.showDuration and " ("..addon:SecondsToTime(duration)..")" or "")
		
		-- start counting timer
		self:SetScript("OnUpdate", onUpdate)
	elseif msg == CLEARED_AFK then
		self:EndAFK()
	end
end

function addon:EndAFK()
	-- stop counting timer
	self:SetScript("OnUpdate", nil)
	
	self.dataobj.text = "Not AFK"
	
	-- print AFK time
	DEFAULT_CHAT_FRAME:AddMessage(format("You were AFK for %s (since %s)", SecondsToTime(duration, nil, true), db.startTimestamp), 1, 1, 0)
	
	db.note = nil
	db.startUptime = nil
	db.startTimestamp = nil
end

-- converts seconds into minutes and seconds
function addon:SecondsToTime(seconds)
	local hours, minutes = 0, 0
	if seconds >= 3600 then
		hours = floor(seconds / 3600)
		minutes = floor((seconds % 3600) / 60)
		seconds = seconds % 60
	elseif seconds >= 60 then
		hours = 0
		minutes = floor(seconds / 60)
		seconds = seconds % 60
	end
	
	return format((hours > 0 and hours..":%.2" or "%").."d:%.2d", minutes, seconds)
end